






/*********************************************************************
  Project Name    :   Mini Dro
  Hardware config :
                      Quadrature X on Timer1 ( PA8 and PA9)
                      Quadrature Y on Timer3 ( PB6 and PB7)
                      Quadrature Z on Timer4 ( PA0 and PA1)
                                  misio PA6   p
                                  mosi  PA7
                                  CS    PA4
                                  SCK   PA5

*********************************************************************/

#include <SPI.h>
#include <GD23STM32.h>
#include "QuadDecoder.h"
#define TFT_PD PB1
#define CS PA4

#define IN_SW_X PB12 //zero X
#define IN_SW_Y PB13 // zero Y
#define IN_SW_Z PB14 // zero Z

#define X !digitalRead(IN_SW_X)
#define Y !digitalRead(IN_SW_Y)
#define Z !digitalRead(IN_SW_Z)

HardwareTimer timer_X(1);
HardwareTimer timer_Y(2);
HardwareTimer timer_Z(4);

QuadDecoder Quad_X(200 , false , false);//wartosc enkodera dla X
QuadDecoder Quad_Y(100, false, false);// wartosc enkodera dla Y
QuadDecoder Quad_Z(100, false, false); //wartosc enkodera dla Z

void setup() {
  pinMode(CS, OUTPUT);
  digitalWrite(CS, HIGH);
 SPI.begin();
 SPI.beginTransaction(SPISettings(21000000, MSBFIRST, SPI_MODE0));
 GD.begin(2);

 GD.wr(REG_PWM_DUTY, 100);// rozjaśnienie ekreanu

  //## configure timer_X as quadrature encoder ##
  pinMode(PA8, INPUT_PULLUP);  
  pinMode(PA9, INPUT_PULLUP);  
  timer_X.setMode(0, TIMER_ENCODER);
  timer_X.pause(); //stop...
  timer_X.setPrescaleFactor(1);
  timer_X.setOverflow(0xFFFF);
  timer_X.setCount(0);
  timer_X.refresh();
  timer_X.setEdgeCounting(TIMER_SMCR_SMS_ENCODER1);
  timer_X.attachInterrupt(0, IT_Overflow_X);
  timer_X.resume();

  //## configure timer_Y as quadrature encoder ##
  pinMode(PB6, INPUT_PULLUP);  
  pinMode(PB7, INPUT_PULLUP);  
  timer_Y.setMode(0, TIMER_ENCODER);
  timer_Y.pause(); //stop...
  timer_Y.setPrescaleFactor(1);
  timer_Y.setOverflow(0xFFFF);
  timer_Y.setCount(0); 
  timer_Y.refresh();
  timer_Y.setEdgeCounting(TIMER_SMCR_SMS_ENCODER1); 
  timer_Y.attachInterrupt(0, IT_Overflow_Y); 
  timer_Y.resume();                 

  //## configure timer_Z as quadrature encoder ##
  pinMode(PA0, INPUT_PULLUP);  
  pinMode(PA1, INPUT_PULLUP); 
  timer_Z.setMode(0, TIMER_ENCODER);
  timer_Z.pause();
  timer_Z.setPrescaleFactor(1);
  timer_Z.setOverflow(0xFFFF);
  timer_Z.setCount(0);        
  timer_Z.refresh();
  timer_Z.setEdgeCounting(TIMER_SMCR_SMS_ENCODER1);
  timer_Z.attachInterrupt(0, IT_Overflow_Z);
  timer_Z.resume();
}
void loop()
{
  Ekran();
  Quad_X.CounterValue(timer_X.getCount());
  Quad_Y.CounterValue(timer_Y.getCount());
  Quad_Z.CounterValue(timer_Z.getCount());

  if ( X == HIGH)  Quad_X.SetZeroActiveMode();
  if ( Y == HIGH)  Quad_Y.SetZeroActiveMode();
  if ( Z == HIGH)  Quad_Z.SetZeroActiveMode();
}

void Ekran()
{
  char buffer_x[16];
  char buffer_y[16];
  char buffer_z[16];
  sprintf (buffer_x, "%8.2f", Quad_X.GetValue());
  sprintf(buffer_y, "%8.2f", Quad_Y.GetValue());
  sprintf(buffer_z, "%8.2f", Quad_Z.GetValue());

  GD.Clear();
  GD.cmd_gradient(128, 84, 11201200, 128, 288, 11201200);
  GD.ColorRGB(0, 0, 0);
  GD.cmd_fgcolor(16744255);
  GD.cmd_button(4, 28, 95, 52, 31, 0, "X");
  GD.cmd_button(4, 110, 95, 52, 31, 0, "Y");
  GD.cmd_button(4, 193, 95, 52, 31, 0, "Z");
  GD.ColorRGB(0, 0, 0);
  GD.cmd_text(200, 28, 31, 0, buffer_x );
  GD.cmd_text(200, 110, 31, 0, buffer_y );
  GD.cmd_text(200, 193, 31, 0, buffer_z);
  GD.swap();
}
void IT_Overflow_X() {
  if (timer_X.getDirection()) {
    Quad_X.OverflowMinus();
  } else {
    Quad_X.OverflowPlus();
  }
}
void IT_Overflow_Y() {
  if (timer_Y.getDirection()) {
    Quad_Y.OverflowMinus();
  } else {
    Quad_Y.OverflowPlus();
  }
}
void IT_Overflow_Z() {
  if (timer_Z.getDirection()) {
    Quad_Z.OverflowMinus();
  } else {
    Quad_Z.OverflowPlus();
  }
}
